@@ -64,9 +64,7 @@ private:
   Lum::Model::ActionRef restartAction;
   Lum::Model::ActionRef stopAction;
   Lum::Model::ActionRef timer;
-  Lum::Model::IntRef    minutes;
-  Lum::Model::IntRef    seconds;
-  size_t                lastMinutes;
+  Lum::Model::SizeTRef  seconds;
   size_t                lastSeconds;
 
   State                 state;
@@ -82,9 +80,7 @@ public:
     restartAction(new Lum::Model::Action()),
     stopAction(new Lum::Model::Action()),
     timer(new Lum::Model::Action()),
-    minutes(new Lum::Model::Int()),
-    seconds(new Lum::Model::Int()),
-    lastMinutes(0),
+    seconds(new Lum::Model::SizeT()),
     lastSeconds(0),
     state(stateUninitialized),
     value(new StopWatch()),
@@ -112,18 +108,9 @@ public:
       }
     }
 
-    restartAction->Disable();
-
-    seconds->SetRange(0,59);
-    minutes->SetRange(0,15);
-
-    minutes->Set(4);
-    seconds->Set(0);
-
     Observe(GetOpenedAction());
     Observe(GetClosedAction());
 
-    Observe(minutes);
     Observe(seconds);
 
     Observe(value->GetClickAction());
@@ -133,6 +120,12 @@ public:
     Observe(restartAction);
     Observe(stopAction);
     Observe(timer);
+
+    // Settings for initial state
+    value->GetClickAction()->Enable();
+    startAction->Disable();
+    restartAction->Disable();
+    stopAction->Disable();
   }
 
   ~MainDialog()
@@ -142,16 +135,19 @@ public:
 
   void SetValue()
   {
-    std::wstring text;
-
-    text=Lum::Base::NumberToWString(minutes->Get());
-    text.append(L":");
-    if (seconds->Get()<10) {
-      text.append(L"0");
+    if (seconds->IsNull()) {
+      value->SetText(L"-:--");
     }
-    text.append(Lum::Base::NumberToWString(seconds->Get()));
+    else {
+      std::wstring text;
+
+      text=Lum::Base::NumberToWString(seconds->Get()/60);
+      text.append(L":");
+      if (seconds->Get()%60<10) {
+        text.append(L"0");
+      }
+      text.append(Lum::Base::NumberToWString(seconds->Get()%60));
 
-    if (value!=NULL) {
       value->SetText(text);
     }
   }
@@ -183,47 +179,51 @@ public:
 
     this->state=state;
 
-    switch (state) {
-    case stateUninitialized:
-      assert(false);
-      break;
-    case stateStoped:
-      startAction->Enable();
-      stopAction->Disable();
-      AllowScreenBlanking(true);
-      break;
-    case stateRunning:
-      restartAction->Enable();
-      AllowScreenBlanking(false);
-    case stateAlerting:
-      startAction->Disable();
-      stopAction->Enable();
-      AllowScreenBlanking(false);
-      break;
-    }
+    // Right
 
     switch (state) {
     case stateUninitialized:
       assert(false);
       break;
     case stateStoped:
+      startAction->Enable();
+      restartAction->Enable();
+      stopAction->Disable();
       value->GetClickAction()->Enable();
+      AllowScreenBlanking(true);
+      break;
+    case stateRunning:
+      restartAction->Enable();
+      value->GetClickAction()->Disable();
+      AllowScreenBlanking(false);
+    case stateAlerting:
+      startAction->Enable();
+      restartAction->Enable();
+      stopAction->Enable();
+      value->GetClickAction()->Disable();
+      AllowScreenBlanking(false);
+      break;
+    }
+
+    // Action
+
+    switch (state) {
+    case stateUninitialized:
+      assert(false);
+      break;
+    case stateStoped:
       value->SetAlarm(false);
       Lum::OS::display->RemoveTimer(timer);
       break;
     case stateRunning:
-      value->GetClickAction()->Disable();
       value->SetAlarm(false);
-      lastMinutes=minutes->Get();
       lastSeconds=seconds->Get();
       alertTime.SetToNow();
-      alertTime.Add(minutes->Get()*60+seconds->Get());
+      alertTime.Add(seconds->Get());
       Lum::OS::display->AddTimer(0,100000,timer);
       break;
     case stateAlerting:
-      value->GetClickAction()->Disable();
       value->SetAlarm(true);
-      minutes->Set(0);
       seconds->Set(0);
       PlayAlert();
       alertTime.SetToNow();
@@ -236,7 +236,6 @@ public:
 
   virtual void PreInit()
   {
-    value->SetText(L"4:00");
     value->SetFlex(true,true);
 
     SetMain(Lum::VPanel::Create(true,true)
@@ -263,8 +262,6 @@ public:
 
     SetMenu(menu);
 
-    SetState(stateStoped);
-
     Dialog::PreInit();
   }
 
@@ -287,36 +284,36 @@ public:
     }
     else if (model==value->GetClickAction() &&
              value->GetClickAction()->IsFinished() &&
-             state==stateStoped) {
+             (state==stateStoped || state==stateUninitialized)) {
       Selector *selector;
 
-      minutes->Push();
-      seconds->Push();
-
-      selector=new Selector(minutes,seconds);
+      selector=new Selector(seconds);
       selector->SetParent(GetWindow());
-      selector->Open();
-      selector->EventLoop();
-      selector->Close();
-
-      minutes->Pop();
-      seconds->Pop();
+      if (selector->Open()) {
+        selector->EventLoop();
+        selector->Close();
+      }
 
       delete selector;
+
+      SetState(stateStoped);
     }
-    else if (model==startAction && startAction->IsFinished()) {
+    else if (model==startAction &&
+             startAction->IsFinished()) {
       SetState(stateRunning);
     }
-    else if (model==restartAction && restartAction->IsFinished()) {
+    else if (model==restartAction &&
+             restartAction->IsFinished()) {
       SetState(stateStoped);
-      minutes->Set(lastMinutes);
       seconds->Set(lastSeconds);
       SetState(stateRunning);
     }
-    else if (model==stopAction && stopAction->IsFinished()) {
+    else if (model==stopAction &&
+             stopAction->IsFinished()) {
       SetState(stateStoped);
     }
-    else if (model==timer && timer->IsFinished()) {
+    else if (model==timer &&
+             timer->IsFinished()) {
       if (state==stateRunning) {
         Lum::Base::SystemTime now;
 
@@ -324,8 +321,7 @@ public:
           SetState(stateAlerting);
         }
         else {
-          minutes->Set((alertTime.GetTime()-now.GetTime())/60);
-          seconds->Set((alertTime.GetTime()-now.GetTime())%60);
+          seconds->Set((alertTime.GetTime()-now.GetTime()));
           Lum::OS::display->AddTimer(0,100000,timer);
         }
       }
@@ -341,7 +337,7 @@ public:
         Lum::OS::display->AddTimer(0,100000,timer);
       }
     }
-    else if (model==minutes || model==seconds) {
+    else if (model==seconds) {
       SetValue();
     }
 
