@@ -32,93 +32,70 @@
 #include <Lum/Table.h>
 #include <Lum/TextValue.h>
 
-Selector::Selector(Lum::Model::Int* minutes, Lum::Model::Int *seconds)
- : value(new Lum::Model::String()),
-   minutes(minutes),
-   seconds(seconds),
+#include "TeeEdit.h"
+
+Selector::Selector(Lum::Model::SizeT *seconds)
+ : seconds(seconds),
    list(new TeasModel(teas,new TeasDataProvider())),
    selection(new Lum::Model::SingleLineSelection()),
    addAction(new Lum::Model::Action()),
-   updateAction(new Lum::Model::Action()),
+   editAction(new Lum::Model::Action()),
    removeAction(new Lum::Model::Action()),
    okAction(new Lum::Model::Action())
 {
-  updateAction->Disable();
-  removeAction->Disable();
+  seconds->Push();
 
-  Observe(minutes);
-  Observe(seconds);
+  editAction->Disable();
+  removeAction->Disable();
 
   Observe(selection);
 
   Observe(addAction);
-  Observe(updateAction);
+  Observe(editAction);
   Observe(removeAction);
-  Observe(okAction);
 
   Observe(GetClosedAction());
+  Observe(okAction);
+
+  list->SetEmptyText(_(L"TBL_TEA_EMPTY",L"(No teas defined)"));
+}
+
+Selector::~Selector()
+{
+  seconds->Pop();
 }
 
 Lum::Object* Selector::GetContent()
 {
-  Lum::Label            *label;
   Lum::Panel            *panel,*horiz;
-  Lum::Slider           *slider;
   Lum::Table            *table;
-  Lum::TextValue        *value;
   Lum::Model::HeaderRef headerModel;
 
   panel=Lum::VPanel::Create(true,true);
 
-  label=Lum::Label::Create(true,true);
-
-  value=Lum::TextValue::Create(this->value,Lum::TextValue::centered,true,false);
-  value->SetFont(Lum::OS::display->GetFont(Lum::OS::Display::fontScaleCaption2));
-  value->SetStyle(Lum::OS::Font::bold);
-  panel->Add(value);
-
-  panel->AddSpace();
-
-  slider=new Lum::HSlider();
-  slider->SetWidth(Lum::Base::Size::stdCharWidth,30);
-  slider->SetFlex(true,false);
-  slider->SetModel(minutes);
-  slider->SetScale(false);
-  label->AddLabel(_(L"LBL_MINUTES",L"Minutes"),slider);
-
-  slider=new Lum::HSlider();
-  slider->SetWidth(Lum::Base::Size::stdCharWidth,30);
-  slider->SetFlex(true,false);
-  slider->SetScale(false);
-  slider->SetModel(seconds);
-
-  label->AddLabel(_(L"LBL_SECONDS",L"Seconds"),slider);
-  panel->Add(label);
-
-  panel->AddSpace();
-
   horiz=Lum::HPanel::Create(true,true);
-  horiz->Add(Lum::VPanel::Create(false,true)
-             ->Add(Lum::Button::Create(_(L"BTN_ADD",L"_Add"),addAction,true,false))
-             ->Add(Lum::Button::Create(_(L"BTN_UPDATE",L"_Update"),updateAction,true,false))
-             ->Add(Lum::Button::Create(_(L"BTN_REMOVE",L"_Remove"),removeAction,true,false)));
-
-  horiz->AddSpace();
 
   headerModel=new Lum::Model::HeaderImpl();
-  headerModel->AddColumn(_(L"TBL_HEADER_TEA",L"Tea"),Lum::Base::Size::stdCharWidth,20,true);
-  headerModel->AddColumn(_(L"TBL_HEADER_TIME",L"Time"),Lum::Base::Size::stdCharWidth,5);
+  headerModel->AddColumn(_(L"TBL_TEA_HEADER_TEA",L"Tea"),Lum::Base::Size::stdCharWidth,25,true);
 
   table=new Lum::Table();
   table->SetFlex(true,true);
-  table->SetShowHeader(true);
-  //table->GetTableView()->SetAutoHFit(true);
+  table->SetShowHeader(false);
+  table->GetTableView()->SetAutoFitColumns(true);
+  table->GetTableView()->SetAutoVSize(true);
   table->SetModel(list);
   table->SetHeaderModel(headerModel);
   table->SetSelection(selection);
   table->SetDoubleClickAction(okAction);
   horiz->Add(table);
 
+  horiz->AddSpace();
+
+  horiz->Add(Lum::VPanel::Create(false,true)
+             ->Add(Lum::Button::Create(_(L"BTN_ADD",L"_Add"),addAction,true,false))
+             ->Add(Lum::Button::Create(_(L"BTN_EDIT",L"_Edit"),editAction,true,false))
+             ->Add(Lum::Button::Create(_(L"BTN_REMOVE",L"_Remove"),removeAction,true,false)));
+
   panel->Add(horiz);
 
   return panel;
@@ -134,66 +111,80 @@ void Selector::Resync(Lum::Base::Model *model, const Lum::Base::ResyncMsg& msg)
   if (model==GetClosedAction() && GetClosedAction()->IsFinished()) {
     Exit();
   }
-  else if (model==okAction && okAction->IsFinished()) {
-    minutes->Save();
-    seconds->Save();
-    Exit();
-  }
-  else if (model==minutes || model==seconds) {
-    std::wstring text;
-
-    text=Lum::Base::NumberToWString(minutes->Get());
-    text.append(L":");
-    if (seconds->Get()<10) {
-      text.append(L"0");
-    }
-    text.append(Lum::Base::NumberToWString(seconds->Get()));
-
-    value->Set(text);
-  }
-  else if (model==selection && dynamic_cast<const Lum::Model::Selection::Selected*>(&msg)!=NULL) {
+  else if (model==selection &&
+           dynamic_cast<const Lum::Model::Selection::Selected*>(&msg)!=NULL) {
     if (selection->HasSelection()) {
-      updateAction->Enable();
+      editAction->Enable();
       removeAction->Enable();
+      okAction->Enable();
 
-      minutes->Set(teas[selection->GetLine()-1].minutes);
-      seconds->Set(teas[selection->GetLine()-1].seconds);
+      seconds->Set(teas[selection->GetLine()-1].minutes*60+
+                   teas[selection->GetLine()-1].seconds);
     }
     else {
-      updateAction->Disable();
+      editAction->Disable();
       removeAction->Disable();
+      okAction->Disable();
     }
   }
-  else if (model==addAction && addAction->IsFinished()) {
+  else if (model==addAction &&
+           addAction->IsFinished()) {
     Lum::Model::StringRef name=new Lum::Model::String(_(L"TEA_NAME_DEFAULT",L"Some tea"));
+    Lum::Model::SizeTRef  time=new Lum::Model::SizeT(4*60);
 
-    if (Lum::Dlg::TextInput::GetText(this->GetWindow(),
-                                     L"",
-                                     _(L"DLG_TEA_NAME_TEXT",
-                                       L"Please enter the name of the tea:"),
-                                       name)) {
+    TeeEdit *teeEdit=new TeeEdit(name,time);
+
+    teeEdit->SetParent(this);
+    if (teeEdit->Open()) {
+      teeEdit->EventLoop();
+      teeEdit->Close();
+    }
+
+    if (teeEdit->Success()) {
       Tea tea;
 
       // The tea list
       tea.name=name->Get();
-      tea.minutes=minutes->Get();
-      tea.seconds=seconds->Get();
+      tea.minutes=time->Get()/60;
+      tea.seconds=time->Get()%60;
 
       // The selection list
       list->Append(tea);
 
       configurationChanged=true;
     }
+
+    delete teeEdit;
   }
-  else if (model==updateAction && updateAction->IsFinished()) {
+  else if (model==editAction && editAction->IsFinished()) {
     assert(selection->HasSelection());
 
-    teas[selection->GetLine()-1].minutes=minutes->Get();
-    teas[selection->GetLine()-1].seconds=seconds->Get();
+    Lum::Model::StringRef name=new Lum::Model::String(teas[selection->GetLine()-1].name);
+    Lum::Model::SizeTRef  time=new Lum::Model::SizeT(teas[selection->GetLine()-1].minutes*60+
+                                                     teas[selection->GetLine()-1].seconds);
 
-    list->RedrawRow(selection->GetLine());
+    TeeEdit *teeEdit=new TeeEdit(name,time);
 
-    configurationChanged=true;
+    teeEdit->SetParent(this);
+    if (teeEdit->Open()) {
+      teeEdit->EventLoop();
+      teeEdit->Close();
+    }
+
+    if (teeEdit->Success()) {
+      teas[selection->GetLine()-1].name=name->Get();
+      teas[selection->GetLine()-1].minutes=time->Get()/60;
+      teas[selection->GetLine()-1].seconds=time->Get()%60;
+
+      seconds->Set(teas[selection->GetLine()-1].minutes*60+
+                   teas[selection->GetLine()-1].seconds);
+
+      list->RedrawRow(selection->GetLine());
+
+      configurationChanged=true;
+    }
+
+    delete teeEdit;
   }
   else if (model==removeAction && removeAction->IsFinished()) {
     assert(selection->HasSelection());
@@ -202,7 +193,11 @@ void Selector::Resync(Lum::Base::Model *model, const Lum::Base::ResyncMsg& msg)
 
     configurationChanged=true;
   }
+  else if (model==okAction && okAction->IsFinished()) {
+    seconds->Save();
+    Exit();
+  }
 
-  Dialog::Resync(model,msg);
+  Lum::Dlg::ActionDialog::Resync(model,msg);
 }
 
