@@ -35,53 +35,127 @@
 typedef Lum::Model::StdRefTable<Tea,std::vector<Tea> > TeasModel;
 typedef Lum::Base::Reference<TeasModel>                TeasModelRef;
 
+class TeaObject : public Lum::Object
+{
+private:
+  Lum::OS::FontRef font;
+  Tea              tea;
+
+public:
+  TeaObject()
+  : font(Lum::OS::display->GetFont())
+  {
+    // no code
+  }
+
+  void SetTea(const Tea& tea)
+  {
+    this->tea=tea;
+  }
+
+  void CalcSize()
+  {
+    minWidth=0;
+    minHeight=font->height;
+
+    width=minWidth;
+    height=minHeight;
+
+    Object::CalcSize();
+  }
+
+  void Draw(int x, int y, size_t w, size_t h)
+  {
+    Object::Draw(x,y,w,h);
+
+    if (!OIntersect(x,y,w,h)) {
+      return;
+    }
+
+    /* --- */
+
+    Lum::OS::DrawInfo *draw=GetDrawInfo();
+
+    draw->PushFont(font);
+    draw->PushForeground(parent->GetTextColor(draw));
+
+    //
+    // Name of Tea
+    //
+
+    draw->DrawString(this->x,
+                     this->y+font->ascent,
+                     tea.name);
+
+    //
+    //
+    // Time of tea
+
+    std::wstring time=Lum::Base::NumberToWString(tea.minutes);
+
+    time.append(L":");
+    if (tea.seconds<10) {
+      time.append(L"0");
+    }
+    time.append(Lum::Base::NumberToWString(tea.seconds));
+
+    draw->DrawString(this->x+width-font->StringWidth(time),
+                     this->y+font->ascent,
+                     time);
+
+    draw->PopFont();
+    draw->PopForeground();
+  }
+
+  static TeaObject* Create(bool horizFlex, bool vertFlex)
+  {
+    TeaObject *object=new TeaObject();
+
+    object->SetFlex(horizFlex,vertFlex);
+
+    return object;
+  }
+};
+
 class TeasDataProvider : public TeasModel::DataProvider
 {
 private:
-  std::wstring GetTimeString(size_t minutes, size_t seconds) const
-  {
-    std::wstring tmp;
-
-    tmp=Lum::Base::NumberToWString(minutes);
-    tmp.append(L":");
-    if (seconds<10) {
-      tmp.append(L"0");
-    }
-    tmp.append(Lum::Base::NumberToWString(seconds));
-
-    return tmp;
-  }
+  TeaObject *object;
 
 public:
-  std::wstring GetString(const TeasModel::Iterator& iter, size_t column) const
+  TeasDataProvider()
+   : object(TeaObject::Create(true,false))
   {
-    switch (column) {
-    case 1:
-      return iter->name;
-    case 2:
-      return GetTimeString(iter->minutes,iter->seconds);
-    default:
-      assert(false);
-    }
+    // no code
+  }
+
+  ~TeasDataProvider()
+  {
+    delete object;
+  }
+
+  Lum::Object* GetObject(const TeasModel::Iterator& iter, size_t column) const
+  {
+    object->SetTea(*iter);
+    return object;
   }
 };
 
 class Selector : public Lum::Dlg::ActionDialog
 {
 private:
-  Lum::Model::StringRef              value;
-  Lum::Model::IntRef                 minutes;
-  Lum::Model::IntRef                 seconds;
+  Lum::Model::SizeTRef               seconds;
   TeasModelRef                       list;
   Lum::Model::SingleLineSelectionRef selection;
 
   Lum::Model::ActionRef              addAction;
-  Lum::Model::ActionRef              updateAction;
+  Lum::Model::ActionRef              editAction;
   Lum::Model::ActionRef              removeAction;
   Lum::Model::ActionRef              okAction;
 
 public:
-  Selector(Lum::Model::Int* minutes, Lum::Model::Int *seconds);
+  Selector(Lum::Model::SizeT *seconds);
+  ~Selector();
 
   Lum::Object* GetContent();
   void GetActions(std::vector<Lum::Dlg::ActionInfo>& actions);
